install.packages("RTextTools")

# creiamo un vettore di frasi positive
pos <- rbind(c("You are awesome and I love you so much", "positive"),
             c("I love traveling, this place is so beautiful", "positive"),
             c("I liked this movie so much", "positive"),
             c("Your pet is adorable", "positive"),
             c("This book is amazing", "positive"),
             c("This is my favourite movie", "positive"),
             c("He is undoubtedly a great actor", "positive"),
             c("Wonderful movie, is really incredible", "positive"),
             c("The director is really experienced", "positive"),
             c("I cannot tell you how much I enjoyed during this traveling", "positive"))

# creiamo un vettore di frasi negative
neg <- rbind(c("I hate and hate and hate. So angry. Die!", "negative"),
             c("Impressed and amazed: you are peerless in your achievement of unparalleled mediocrity", "negative"),
             c("This is possibly the worst movie even made", "negative"),
             c("I don t like it, is boring", "negative"),
             c("Don t waste your time and money", "negative"),
             c("Very bad acting, very boring, I am so disappointed!", "negative"),
             c("It is garbage, it is so trash", "negative"),
             c("This book is simply terrible, I am horrified", "negative"),
             c("So many thing are so idiotic that it pissed me off", "negative"),
             c("I thought the film was totally awful, bad script and acting", "negative"))

# creiamo un vettore di test che contiene sia frasi positive che negative
test <- rbind(c("It was a great experience", "positive"),
              c("Bad acting, bad music and especially bad taste", "negative"),
              c("This is the best movie created so far", "positive"),
              c("It's a waste of time", "negative"),
              c("Well directed, well produced, I like it", "positive"),
              c("I love the photography and the script is amazing", "positive"),
              c("Truly disappointed from this product", "negative"),
              c("All the characters are beautifully executed", "positive"),
              c("Unfortunately I didn't like it at all", "negative"),
              c("Horrid acting and mediocre script, I'm unhappy", "negative"))


# creiamo un oggetto unico
tot <- rbind(pos, neg, test)

# carichiamo il pacchetto RTextTools
library(RTextTools)

# creiamo la DTM
dtm <- create_matrix(tot[,1], language="english", removeStopwords=TRUE, removeNumbers=TRUE, stemWords=TRUE, removeSparseTerms=.998)

# convertiamo in matrice
mat <- as.matrix(dtm)

# creiamo il classificatore Naive Bayes
library(e1071)
classifier <- naiveBayes(mat[1:20,], as.factor(tot[1:20,2]) )

# prediciamo i risultati e verifichiamoli
predicted <- predict(classifier, mat[21:30,])

table(tot[21:30, 2], predicted)

recall_accuracy(tot[21:30, 2], predicted)

# costruiamo il container dove specifichiamo la variabile di risposta, il train set e il test set
container <- create_container(mat, as.numeric(as.factor(tot[,2])), trainSize=1:20, testSize=21:30,virgin=FALSE)

models <- train_models(container, algorithms=c("MAXENT" , "SVM", "RF", "BAGGING", "TREE"))

results <- classify_models(container, models)

# misuriamo la precisione sui vari modelli
table(as.numeric(as.factor(tot[21:30, 2])), results[,"FORESTS_LABEL"])
table(as.numeric(as.factor(tot[21:30, 2])), results[,"MAXENTROPY_LABEL"])

recall_accuracy(as.numeric(as.factor(tot[21:30, 2])), results[,"FORESTS_LABEL"])
recall_accuracy(as.numeric(as.factor(tot[21:30, 2])), results[,"MAXENTROPY_LABEL"])
recall_accuracy(as.numeric(as.factor(tot[21:30, 2])), results[,"TREE_LABEL"])
recall_accuracy(as.numeric(as.factor(tot[21:30, 2])), results[,"BAGGING_LABEL"])
recall_accuracy(as.numeric(as.factor(tot[21:30, 2])), results[,"SVM_LABEL"])

# mettiamo a confronto i risultati
analytics <- create_analytics(container, results)
summary(analytics)
head(analytics@document_summary)
analytics@ensemble_summary


n=4
set.seed(12345)
cross_validate(container,n,"MAXENT")
cross_validate(container,n,"TREE")
cross_validate(container,n,"SVM")
cross_validate(container,n,"RF")



##### Esempio con un dataset più corposo #####

rec <- read.csv("rec.csv", encoding = "UTF-8", stringsAsFactors = FALSE)
sent <- c(rep("pos",500), rep("neg", 500))

df <- cbind(rec, sent)


set.seed(12345)
random <- runif(nrow(df))
df2 <- df[order(random),]

# creiamo la DTM
dtm <- create_matrix(df2[,1], language="english", removeStopwords=TRUE, removeNumbers=TRUE, stemWords=TRUE, removeSparseTerms=.998)

# convertiamo in matrice
mat <- as.matrix(dtm)

# creiamo il classificatore Naive Bayes
library(e1071)
classifier <- naiveBayes(mat[1:1000,], as.factor(df2[1:1000,2]) )

# prediciamo i risultati e verifichiamoli
predicted <- predict(classifier, df2[1:1000,2])

table(df2[1:1000,2], predicted)

recall_accuracy(df2[1:1000,2], predicted)

# costruiamo il container dove specifichiamo la variabile di risposta, il train set e il test set
container <- create_container(mat, as.numeric(as.factor(df2[1:1000,2])), trainSize=1:800, testSize=801:1000,virgin=FALSE)

models <- train_models(container, algorithms=c("MAXENT" , "SVM", "RF", "BAGGING", "TREE"))

results <- classify_models(container, models)

# misuriamo la precisione sui vari modelli
table(as.numeric(as.factor(df2[801:1000,2])), results[,"FORESTS_LABEL"])
table(as.numeric(as.factor(df2[801:1000,2])), results[,"MAXENTROPY_LABEL"])

recall_accuracy(as.numeric(as.factor(df2[801:1000,2])), results[,"FORESTS_LABEL"])
recall_accuracy(as.numeric(as.factor(df2[801:1000,2])), results[,"MAXENTROPY_LABEL"])
recall_accuracy(as.numeric(as.factor(df2[801:1000,2])), results[,"TREE_LABEL"])
recall_accuracy(as.numeric(as.factor(df2[801:1000,2])), results[,"BAGGING_LABEL"])
recall_accuracy(as.numeric(as.factor(df2[801:1000,2])), results[,"SVM_LABEL"])

# mettiamo a confronto i risultati
analytics <- create_analytics(container, results)
summary(analytics)
head(analytics@document_summary)
analytics@ensemble_summary


n=4
set.seed(12345)
cross_validate(container,n,"MAXENT")
cross_validate(container,n,"TREE")
cross_validate(container,n,"SVM")
cross_validate(container,n,"RF")

# possiamo infine scrivere i risultati in un file in questo modo
write.csv(analytics@document_summary, "Summary.csv")


